/*
 * Copyright (C) 2023-2024
 * Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package config

import (
	"os"
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestLoadConfig(t *testing.T) {
	t.Run("should load config from file", func(t *testing.T) {
		config := LoadConfig()

		expectedConfig := Config{}
		expectedConfig.Server.Port = 8080
		expectedConfig.Grpc.Url = "localhost:50051"
		expectedConfig.Grpc.Mock = false
		expectedConfig.Logging.Level = "DEBUG"

		assert.Equal(t, expectedConfig, config)
	})

	t.Run("should load config from env", func(t *testing.T) {
		envVars := map[string]string{
			"SERVER_PORT":   "9090",
			"GRPC_URL":      "localhost:99999",
			"LOGGING_LEVEL": "ERROR",
		}

		for key, value := range envVars {
			assert.NoError(t, os.Setenv(key, value), "Setenv "+key+" should not return an error")
		}

		config := LoadConfig("testdata/empty_test_config.yml")

		for key := range envVars {
			assert.NoError(t, os.Unsetenv(key), "Unsetenv "+key+" should not return an error")
		}

		expectedConfig := Config{}
		expectedConfig.Server.Port = 9090
		expectedConfig.Grpc.Url = "localhost:99999"
		expectedConfig.Grpc.Mock = false
		expectedConfig.Logging.Level = "ERROR"

		assert.Equal(t, expectedConfig, config)
	})

	t.Run("should overwrite config with env", func(t *testing.T) {
		assert.NoError(t, os.Setenv("SERVER_PORT", "9090"), "Setenv SERVER_PORT should not return an error")

		config := LoadConfig()

		assert.NoError(t, os.Unsetenv("SERVER_PORT"), "Unsetenv SERVER_PORT should not return an error")

		expectedConfig := Config{}
		expectedConfig.Server.Port = 9090
		expectedConfig.Grpc.Url = "localhost:50051"
		expectedConfig.Grpc.Mock = false
		expectedConfig.Logging.Level = "DEBUG"

		assert.Equal(t, expectedConfig, config)
	})
}

func TestValidateGrpcUrl(t *testing.T) {
	testData := []struct {
		grpcUrl        string
		expectedResult bool
	}{
		{"localhost:56789", true},
		{"test.example.com:20", true},
		{"invalidUrl", false},
		{"%test%:2345", false},
		{"localhost:234567", false},
		{"example.com:", false},
		{":6789", false},
		{"example..com:8080", false},
		{"example.com:abc", false},
	}

	for _, data := range testData {
		result := ValidateGrpcUrl(data.grpcUrl)

		assert.Equal(t, data.expectedResult, result)
	}
}
